#ifndef __ATMEGA328P__
#define __ATMEGA328P__

/*
	TIMER0 - @250kHz LCD1602
	TIMER1 - 16-bit Timer
	TIMER2 - @250kHz/250 = @1 mS - used for system time count (glTime), buttons routines
*/


/*
	Arduino	D0...D7  => PD0...PD7
			D8...D13 => PB0...PB5
			A0...A5  => PC0...PC5

	PD0/RX
	PD1/TX
	PD2/INT0	LCD/RS
	PD3/INT1	LCD/EN
	PD4			LCD/D4
	PD5			LCD/D5
	PD6			LCD/D6
	PD7			LCD/D7
	PB0			A4988/~EN
	PB1			A4988/STEP
	PB2/SS		A4988/DIR
	PB3/MOSI
	PB4/MISO
	PB5/SCK
	PC0
	PC1			BUZZER
	PC2
	PC3
	PC4/SDA
	PC5/SCL
	ADC6		BUTTONS
	ADC7
*/

/*
	 ------	VCC
	/  ---- MOS
	| /  -- GND
	| | /
	| | |
	o o o
  ->o o o
	| | |
	| | \
	| \  --	RES
	\  ----	SCK
	 ------	MIS
*/

/*
	PB0	0
	PB1	1
	PB2	2	(SPI SS)
	PB3	3	SPI MOSO
	PB4	4	SPI MISO
	PB5	5	SPI SCK
	PB6	6	-
	PB7	7	-

	PC0	8
	PC1	9
	PC2	A
	PC3	B
	PC4	C	I2C SDA
	PC5	D	I2C SCL
	PC6		(RESET)
	PC7 -

	PD0	10	UART RX
	PD1	11	UART TX
	PD2	12
	PD3	13
	PD4	14
	PD5	15
	PD6	16
	PD7	17
*/




// Type definitions

typedef   signed char     int8_t;
typedef   signed short    int16_t;
typedef   signed long     int32_t;
typedef unsigned char     uint8_t;
typedef unsigned short    uint16_t;
typedef unsigned long     uint32_t;
typedef int32_t		s32;
typedef int16_t		s16;
typedef int8_t		s8;
typedef uint32_t	u32;
typedef uint16_t	u16;
typedef uint8_t		u8;
typedef u8* pu8;
typedef u8 __flash u8f;
typedef u16 __flash u16f;
typedef u8f* u8s;
typedef u8s __flash u8sf;

// Assembler

#define CLI()		{asm("CLI\n");}		// disable interrupts
#define SEI()		{asm("SEI\n");}		// enable interrupts
#define NOP()		{asm("NOP\n");}		// no operation
#define WDR() 		{asm("WDR\n");}
#define SLEEP()		do { *SMCR=SLEEP_MODE_EXT_STANDBY|SE; asm("SLEEP\n"); *SMCR=0; } while(0)
//#define _CRIT(a)	do { u8 gl_bOldSREG = *SREG; CLI(); a; *SREG = gl_bOldSREG; } while(0)
#define _CRIT(a)	do { CLI(); a; SEI(); } while(0)

#define NULL ((void*)0)
typedef enum {FALSE = 0, TRUE = !FALSE} bool;

#define MAX(a,b) (((a)>(b))?(a):(b))
#define MIN(a,b) (((a)<(b))?(a):(b))

#define SWAP(a,b) do { u8 t; t=a; a=b; b=t; } while(0)

#define PSTR(s) {static u8f __c[]=(s); &__c[0];}

#define PB(a) ((u8*)&a)
#define PW(a) ((u16*)&a)
#define PDW(a) ((u32*)&a)

#define B(a) (*PB(a))
#define W(a) (*PW(a))
#define DW(a) (*PDW(a))

#define HINIBBLE(a) (((a)>>4)&0x0F)
#define LONIBBLE(a) ( (a)    &0x0F)
#define HIBYTE(a) (*(PB(a)+1))
#define LOBYTE(a) B(a)
#define HIWORD(a) (*(PW(a)+1))
#define LOWORD(a) W(a)

#define BCD_U(a)	(LONIBBLE(a)+10*HINIBBLE(a))
#define BCD_P(a)	(((a)%10)+(((a)/10)<<4))

#define BIT(n) (1<<(n))
#define BV(b,sh,cnt) (((b)>>sh)&((1<<(cnt))-1))


// Interrupts

#define IRQ_RESET	(0x00)	// RESET External Pin, Power-on Reset, Brown-out Reset and Watchdog System Reset
#define IRQ_INT0	(0x04)	// INT0 External Interrupt Request 0
#define IRQ_INT1	(0x08)	// INT1 External Interrupt Request 1
#define IRQ_PCINT0	(0x0C)	// PCINT0 Pin Change Interrupt Request 0
#define IRQ_PCINT1	(0x10)	// PCINT1 Pin Change Interrupt Request 1
#define IRQ_PCINT2	(0x14)	// PCINT2 Pin Change Interrupt Request 2
#define IRQ_WDT		(0x18)	// WDT Watchdog Time-out Interrupt
#define IRQ_TIM2CA	(0x1C)	// TIMER2 COMPA Timer/Counter2 Compare Match A
#define IRQ_TIM2CB	(0x20)	// TIMER2 COMPB Timer/Counter2 Compare Match B
#define IRQ_TIM2OVF	(0x24)	// TIMER2 OVF Timer/Counter2 Overflow
#define IRQ_TIM1CAP	(0x28)	// TIMER1 CAPT Timer/Counter1 Capture Event
#define IRQ_TIM1CA	(0x2C)	// TIMER1 COMPA Timer/Counter1 Compare Match A
#define IRQ_TIM1CB	(0x30)	// TIMER1 COMPB Timer/Coutner1 Compare Match B
#define IRQ_TIM1OVF	(0x34)	// TIMER1 OVF Timer/Counter1 Overflow
#define IRQ_TIM0CA	(0x38)	// TIMER0 COMPA Timer/Counter0 Compare Match A
#define IRQ_TIM0CB	(0x3C)	// TIMER0 COMPB Timer/Counter0 Compare Match B
#define IRQ_TIM0OVF	(0x40)	// TIMER0 OVF Timer/Counter0 Overflow
#define IRQ_SPI		(0x44)	// SPI, STC SPI Serial Transfer Complete
#define IRQ_USARTRX	(0x48)	// USART, RX USART Rx Complete
#define IRQ_USARTEM	(0x4C)	// USART, UDRE USART, Data Register Empty
#define IRQ_USARTTX	(0x50)	// USART, TX USART, Tx Complete
#define IRQ_ADC		(0x54)	// ADC ADC Conversion Complete
#define IRQ_EERDY	(0x58)	// EE READY EEPROM Ready
#define IRQ_ANCOMP	(0x5C)	// ANALOG COMP Analog Comparator
#define IRQ_TWI		(0x60)	// TWI 2-wire Serial Interface
#define IRQ_SPMRDY	(0x64)	// SPM READY Store Program Memory Ready


#define PRTB	((u8*)0x23)	// IDR DDR ODR		PIN DDR PORT
#define PRTC	((u8*)0x26)	// IDR DDR ODR		PIN DDR PORT
#define PRTD	((u8*)0x29)	// IDR DDR ODR		PIN DDR PORT


#define TIFR0	((u8*)0x35)	// x : x : x : x : x : OCF0B : OCF0A : TOV0
#define TIFR1	((u8*)0x36)	// x : x : ICF1 : x : x : OCF1B : OCF1A : TOV1 (page 139)
#define TIFR2	((u8*)0x37)	// x : x : x : x : x : OCF2B : OCF2A : TOV2 (page 163)
#define PCIFR	((u8*)0x3B)	// x : x : x : x : x : PCIF2 : PCIF1 : PCIF0
#define EIFR	((u8*)0x3C)	// x : x : x : x : x : x : INTF1 : INTF0 (page 72)
#define EIMSK	((u8*)0x3D)	// x : x : x : x : x : x : INT1 : INT0 (page 72)
#define GPIOR0	((u8*)0x3E)	// General Purpose I/O Register 0 (page 25)

#define EECR	((u8*)0x3F)	// x : x : EEPM1 : EEPM0 : EERIE : EEMPE : EEPE : EERE (page 21)
#define				EEPM_E			BIT(4)	// EEPROM Programming Mode Bits 1:0=01, Erase Only (1.8ms)
#define				EEPM_W			BIT(5)	// EEPROM Programming Mode Bits 1:0=10, Write Only (1.8ms)
#define				EEPM_EW			0				// EEPROM Programming Mode Bits 1:0=00, Erase/Write (3.4ms)
#define				EERIE				BIT(3)	// EEPROM Ready Interrupt Enable
#define				EEMPE				BIT(2)	// EEPROM Master Write Enable
#define				EEPE				BIT(1)	// EEPROM Write Enable
#define				EERE				BIT(0)	// EEPROM Read Enable
#define EEDR	((u8*)0x40)	// EEPROM Data Register (page 21)
#define EEAR	((u16*)0x41)// EEPROM Address Register High:Low Byte (page 21)
#define EEARL	((u8*)0x41)	// EEPROM Address Register Low Byte (page 21)
#define EEARH	((u8*)0x42)	// EEPROM Address Register High Byte (page 21)

#define GTCCR	((u8*)0x43)	// TSM : x : x : x : x : x : PSRASY : PSRSYNC (page 143/165)

#define TCCR0A	((u8*)0x44)	// COM0A1 : COM0A0 : COM0B1 : COM0B0 : x : x : WGM01 : WGM00
#define						   					 		  				   		   WGM00			BIT(0)
#define						   					 		  				   WGM01					BIT(1)
#define				OC0A_DIS																		0x00
#define				OC0A_TGL																		0x40
#define				OC0A_CLR																		0x80
#define				OC0A_SET																		0xC0
#define				OC0B_DIS																		0x00
#define				OC0B_TGL																		0x10
#define				OC0B_CLR																		0x20
#define				OC0B_SET																		0x30
#define				TIM0_A_NORM_FF																	0x00
#define				TIM0_A_PCPWM_FF																	0x01
#define				TIM0_A_CTC_OCR0A																0x02
#define				TIM0_A_FPWM_FF																	0x03
#define				TIM0_A_PCPWM_OCR0A																0x05
#define				TIM0_A_FPWM_OCR0A																0x07
#define TCCR0B	((u8*)0x45)	// FOC0A : FOC0B : x : x : WGM02 : CS02 : CS01 : CS00
#define						   FOC0A																BIT(7)
#define						   		   FOC0B														BIT(6)
#define						   					 	   WGM02										BIT(3)
#define				TIM0_B_NORM_FF																0x00
#define				TIM0_B_PCPWM_FF																0x00
#define				TIM0_B_CTC_OCR0A															0x00
#define				TIM0_B_FPWM_FF																0x00
#define				TIM0_B_PCPWM_OCR0A														(WGM02)
#define				TIM0_B_FPWM_OCR0A															(WGM02)
#define				TIM0_STOP																			0	// 0
#define				TIM0_16M																			1	// 1				62.5ns
#define				TIM0_2M																				2	// /8				0.5us
#define				TIM0_250K																			3	// /64			4 us
#define				TIM0_62500																		4	//	/256		16us
#define				TIM0_15625																		5	//	/1024		64us
#define				TIM0_T0_FALL																	6
#define				TIM0_T0_RISE																	7
#define TCNT0	((u8*)0x46)	// Timer/Counter0 (8-bit)
#define OCR0A	((u8*)0x47)	// Timer/Counter0 Output Compare Register A
#define OCR0B	((u8*)0x48)	// Timer/Counter0 Output Compare Register B

#define GPIOR1	((u8*)0x4A)	// General Purpose I/O Register 1 (page 25)
#define GPIOR2	((u8*)0x4B)	// General Purpose I/O Register 2 (page 25)

//	SPI
#define SPCR	((u8*)0x4C)	// SPIE : SPE : DORD : MSTR : CPOL : CPHA : SPR1 : SPR0			// SPI Control Register (page 173)
#define                        SPIE													BIT(7)	// enable IRQ_SPI on SPIF bit in the SPSR set
#define                               SPE											BIT(6)	// enable SPI
#define                                     DORD									BIT(5)	// 0: MSB...LSB / 1: LSB...MSB
#define                                            MSTR								BIT(4)	// 0: Slave / 1: Master
#define                                                   CPOL						BIT(3)	// 0: SCK __|^| / 1: SCK ^^|_|
#define                                                          CPHA				BIT(2)	// 0: Sample/Setup / 1: Setup/Sample
#define				SPI_MODE0														0x00
#define				SPI_MODE1														(CPOL)
#define				SPI_MODE2														(CPHA)
#define				SPI_MODE3														(CPOL|CPHA)
#define				SPI_4M															0x00
#define				SPI_1M															0x01
#define				SPI_250K														0x02
#define				SPI_125K														0x03
#define SPSR	((u8*)0x4D)	// SPIF : WCOL : x : x : x : x : x : SPI2X						// SPI Status Register (page 174)
#define                        SPIF													BIT(7)	// 1 when SPI transfer complete, to clear: read SPSR, read SPDR
#define                               WCOL											BIT(6)	// Write COLlision Flag
#define                                                          SPI2X				BIT(0)	// Double SPI Speed Bit
#define SPDR	((u8*)0x4E)																	// SPI Data Register (page 175)

#define ACSR	((u8*)0x50)	// ACD : ACBG : ACO : ACI : ACIE : ACIC : ACIS1 : ACIS0 (page 247)
#define						   ACD													BIT(7)	// Analog Comparator Disable

// Atmega328p 5V 16MHz power consumption, mA
// Active			9.60
// Idle				2.30
// Power down		0.15uA
//	UART			0.18
//	I2C				0.38
//	Timer2			0.41
//	Timer1			0.33
//	Timer0			0.11
//	SPI				0.36
//	ADC				0.51





#define SMCR	((u8*)0x53)	// x : x : x : x : SM2 : SM1 : SM0 : SE (page 40)
#define															 SE					BIT(0)
#define				SLEEP_MODE_IDLE													0x00
#define				SLEEP_MODE_ADC													0x02
#define				SLEEP_MODE_PWR_DOWN												0x04	// ext. osc. - Timer2 -
#define				SLEEP_MODE_PWR_SAVE												0x06	// ext. osc. - Timer2 +
#define				SLEEP_MODE_STANDBY												0x0C	// ext. osc. + Timer2 -
#define				SLEEP_MODE_EXT_STANDBY											0x0E	// ext. osc. + Timer2 +
#define MCUSR	((u8*)0x54)	// x : x : x : x : WDRF : BORF : EXTRF : PORF (page 54)
#define										   WDRF									BIT(3)
#define												  BORF							BIT(2)
#define														 EXTRF					BIT(1)
#define																 PORF			BIT(0)
#define MCUCR	((u8*)0x55)	// x : BODS : BODSE : PUD : x : x : IVSEL : IVCE (page 44/68/92)
#define SPMCSR	((u8*)0x57)	// SPMIE : RWWSB. : x : RWWSRE. : BLBSET : PGWRT : PGERS : SELFPRGEN (page 292)
#define SPL		((u8*)0x5D)	// SP7 : SP6 : SP5 : SP4 : SP3 : SP2 : SP1 : SP0 (page 12)
#define SPH		((u8*)0x5E)	// x : x : x : x : x : SP10 : SP9 : SP8 (page 12)
#define SREG	((u8*)0x5F)	// I : T : H : S : V : N : Z : C (page 9)
#define WDTCSR	((u8*)0x60)	// WDIF : WDIE : WDP3 : WDCE : WDE : WDP2 : WDP1 : WDP0 (page 54)
#define CLKPR	((u8*)0x61)	// CLKPCE : x : x : x : CLKPS3 : CLKPS2 : CLKPS1 : CLKPS0 (page 37)
#define PRR		((u8*)0x64)	// PRTWI : PRTIM2 : PRTIM0 : x : PRTIM1 : PRSPI : PRUSART0 : PRADC (page 42)
#define						   PRTWI															BIT(7)
#define								   PRTIM2													BIT(6)
#define											PRTIM0											BIT(5)
#define														 PRTIM1								BIT(3)
#define																  PRSPI						BIT(2)
#define																		  PRUSART0			BIT(1)
#define																					 PRADC	BIT(0)
#define OSCCAL	((u8*)0x66)	// Oscillator Calibration Register (page 37)
#define PCICR	((u8*)0x68)	// x : x : x : x : x : PCIE2 : PCIE1 : PCIE0
#define											   PCIE2										BIT(2)	// PCINT23..16 - Port D
#define													   PCIE1								BIT(1)	// PCINT15..8  - Port C
#define															   PCIE0						BIT(0)	// PCINT7..0   - Port B

#define EICRA	((u8*)0x69)	// x : x : x : x : ISC11 : ISC10 : ISC01 : ISC00 (page 71)

#define PCMSK0	((u8*)0x6B)	// PCINT7 : PCINT6 : PCINT5 : PCINT4 : PCINT3 : PCINT2 : PCINT1 : PCINT0 (page 74)			- Port B
#define PCMSK1	((u8*)0x6C)	// x : PCINT14 : PCINT13 : PCINT12 : PCINT11 : PCINT10 : PCINT9 : PCINT8 (page 74)			- Port C
#define PCMSK2	((u8*)0x6D)	// PCINT23 : PCINT22 : PCINT21 : PCINT20 : PCINT19 : PCINT18 : PCINT17 : PCINT16 (page 74)	- Port D

#define TIMSK0	((u8*)0x6E)	// x : x : x : x : x : OCIE0B : OCIE0A : TOIE0 (page 111)
#define											   OCIE0B							BIT(2)
#define													    OCIE0A					BIT(1)
#define															     TOIE0			BIT(0)
#define TIMSK1	((u8*)0x6F)	// x : x : ICIE1 : x : x : OCIE1B : OCIE1A : TOIE1 (page 139)
#define								   ICIE1										BIT(5)
#define												   OCIE1B						BIT(2)
#define															OCIE1A				BIT(1)
#define																	 TOIE1		BIT(0)
#define TIMSK2	((u8*)0x70)	// x : x : x : x : x : OCIE2B : OCIE2A : TOIE2 (page 163)
#define											   OCIE2B							BIT(2)
#define													    OCIE2A					BIT(1)
#define															     TOIE2			BIT(0)

#define ADC		((u16*)0x78)// ADC Data Register Low byte (page 265)
#define ADCL	((u8*)0x78)	// ADC Data Register Low byte (page 265)
#define ADCH	((u8*)0x79)	// ADC Data Register High byte (page 265)
#define ADCSRA	((u8*)0x7A)	// ADEN : ADSC : ADATE : ADIF : ADIE : ADPS2 : ADPS1 : ADPS0 (page 263)
#define 					   ADEN													BIT(7)
#define								  ADSC											BIT(6)
#define				ADC_ZERO														0
#define				ADC_8M															1
#define				ADC_4M															2
#define				ADC_2M															3
#define				ADC_1M															4
#define				ADC_500K														5
#define				ADC_250K														6
#define				ADC_125K														7
#define ADCSRB	((u8*)0x7B)	// x : ACME : x : x : x : ADTS2 : ADTS1 : ADTS0 (page 265)
#define ADMUX	((u8*)0x7C)	// REFS1 : REFS0 : ADLAR : x : MUX3 : MUX2 : MUX1 : MUX0 (page 262)
#define										   ADLAR								BIT(5)
#define 			ADCREF_AREF														0x00		// reference ADC voltage = AREF pin, internal Vref turned off
#define				ADCREF_AVCC														0x40		// reference ADC voltage = AVCC (-> AREF pin)
#define				ADCREF_BGAP														0xC0		// reference ADC voltage = Vref Banggap 1.1V (-> AREF pin)
#define 			ADC0															0x00
#define 			ADC1															0x01
#define 			ADC2															0x02
#define 			ADC3															0x03
#define 			ADC4															0x04
#define 			ADC5															0x05
#define 			ADC6															0x06
#define 			ADC7															0x07
#define 			ADC_TEMP														0x08
#define 			ADC_BGAP														0x0E
#define 			ADC_GND															0x0F

#define DIDR0	((u8*)0x7E)	// x : x : ADC5D : ADC4D : ADC3D : ADC2D : ADC1D : ADC0D (page 266)
#define DIDR1	((u8*)0x7F)	// x : x : x : x : x : x : AIN1D : AIN0D (page 249)

#define TCCR1A	((u8*)0x80)	// COM1A1 : COM1A0 : COM1B1 : COM1B0 : x : x : WGM11 : WGM10 (page 134)
#define				OC1A_DIS																		0x00
#define				OC1A_TGL																		0x40
#define				OC1A_CLR																		0x80
#define				OC1A_SET																		0xC0
#define				OC1B_DIS																		0x00
#define				OC1B_TGL																		0x10
#define				OC1B_CLR																		0x20
#define				OC1B_SET																		0x30
#define						   					 		  				   WGM11					BIT(1)
#define						   					 		  				   		   WGM10			BIT(0)
#define TCCR1B	((u8*)0x81)	// ICNC1 : ICES1 : x : WGM13 : WGM12 : CS12 : CS11 : CS10 (page 136)
#define						   ICNC1																BIT(7)
#define								   ICES1														BIT(6)
#define						   					   WGM13											BIT(4)
#define						   					 		   WGM12									BIT(3)
#define				TIM1_STOP																		0
#define				TIM1_16M																		1
#define				TIM1_2M																			2
#define				TIM1_250K																		3
#define				TIM1_62500																		4
#define				TIM1_15625																		5
#define				TIM1_T1_FALL																	6
#define				TIM1_T1_RISE																	7
#define TCCR1C	((u8*)0x82)	// FOC1A : FOC1B : x : x : x : x : x : x (page 137)
#define						   FOC1A																BIT(7)
#define						   		   FOC1B														BIT(6)
#define TCNT1	((u16*)0x84)
#define TCNT1L	((u8*)0x84)	// Timer/Counter1 - Counter Register Low Byte (page 138)
#define TCNT1H	((u8*)0x85)	// Timer/Counter1 - Counter Register High Byte (page 138)
#define ICR1L	((u8*)0x86)	// Timer/Counter1 - Input Capture Register Low Byte (page 138)
#define ICR1H	((u8*)0x87)	// Timer/Counter1 - Input Capture Register High Byte (page 138)
#define OCR1AL	((u8*)0x88)	// Timer/Counter1 - Output Compare Register A Low Byte (page 138)
#define OCR1AH	((u8*)0x89)	// Timer/Counter1 - Output Compare Register A High Byte (page 138)
#define OCR1BL	((u8*)0x8A)	// Timer/Counter1 - Output Compare Register B Low Byte (page 138)
#define OCR1BH	((u8*)0x8B)	// Timer/Counter1 - Output Compare Register B High Byte (page 138)

#define TCCR2A	((u8*)0xB0)	// COM2A1 : COM2A0 : COM2B1 : COM2B0 : x : x : WGM21 : WGM20 (page 158)
#define				OC2A_DIS																		0x00
#define				OC2A_TGL																		0x40
#define				OC2A_CLR																		0x80
#define				OC2A_SET																		0xC0
#define				OC2B_DIS																		0x00
#define				OC2B_TGL																		0x10
#define				OC2B_CLR																		0x20
#define				OC2B_SET																		0x30
#define				TIM2_A_NORM_FF																	0x00
#define				TIM2_A_PCPWM_FF																	0x01
#define				TIM2_A_CTC_OCR2A																0x02
#define				TIM2_A_FPWM_FF																	0x03
#define				TIM2_A_PCPWM_OCR2A																0x05
#define				TIM2_A_FPWM_OCR2A																0x07
#define TCCR2B	((u8*)0xB1)	// FOC2A : FOC2B : x : x : WGM22 : CS22 : CS21 : CS20 (page 161)
#define						   FOC2A																BIT(7)
#define						   		   FOC2B														BIT(6)
#define						   					 	   WGM22										BIT(3)
#define														   CS22									BIT(2)
#define														   		  CS21							BIT(1)
#define														   				 CS20					BIT(0)
#define				TIM2_B_NORM_FF																	0x00
#define				TIM2_B_PCPWM_FF																	0x00
#define				TIM2_B_CTC_OCR2A																0x00
#define				TIM2_B_FPWM_FF																	0x00
#define				TIM2_B_PCPWM_OCR2A																(WGM22)
#define				TIM2_B_FPWM_OCR2A																(WGM22)
#define							TIM2_STOP															0	//
#define							TIM2_16M															1	//	/1 downto 62.5kHz
#define							TIM2_2M																2	//	/8 7812.5Hz
#define							TIM2_500K															3	//	/32 1953Hz
#define							TIM2_250K															4	//	/64 977Hz
#define							TIM2_125K															5	//	/128 488Hz
#define							TIM2_62500															6	//	/256 244Hz
#define							TIM2_15625															7	//	/1024 61Hz
#define TCNT2	((u8*)0xB2)	// Timer/Counter2 (8-bit) (page 162)
#define OCR2A	((u8*)0xB3)	// Timer/Counter2 Output Compare Register A (page 162)
#define OCR2B	((u8*)0xB4)	// Timer/Counter2 Output Compare Register B (page 162)
#define ASSR	((u8*)0xB6)	// x : EXCLK : AS2 : TCN2UB : OCR2AUB : OCR2BUB : TCR2AUB : TCR2BUB (page 164)

#define TWBR	((u8*)0xB8)	// 2-wire Serial Interface Bit Rate Register (page 241)
#define				TWI_BR(ClockSpeed,Prescale)			((16000000L/ClockSpeed-16)/2/Prescale)
#define TWSR	((u8*)0xB9)	// TWS7 : TWS6 : TWS5 : TWS4 : TWS3 : x : TWPS1 : TWPS0 (page 243)
#define				TWI_PRESCALE_1															0x00
#define				TWI_PRESCALE_4															0x01
#define				TWI_PRESCALE_16															0x02
#define				TWI_PRESCALE_64															0x03
#define				TWI_STATUS_MASK															0xF8
#define				TWI_ST_START_OK															0x08
#define				TWI_ST_NOT_ACK_SLAW														0x20
#define				TWI_ST_NOT_ACK_DATA														0x30
#define				TWI_ST_ARB_LOST															0x38
#define				TWI_ST_NOT_ACK_SLAR														0x48

/*
0x08	T/R A START condition has been transmitted
0x10	T/R A repeated START condition has been transmitted
0x18	T   SLA+W has been transmitted; ACK has been received
0x20	T   SLA+W has been transmitted; NOT ACK has been received
0x28	T   Data byte has been transmitted; ACK has been received
0x30	T   Data byte has been transmitted; NOT ACK has been received
0x38	T/R Arbitration lost in SLA+W/R or data bytes/NOT ACK bit
0x40	R   SLA+R has been transmitted; ACK has been received
0x48	R   SLA+R has been transmitted; NOT ACK has been received
0x50	R   Data byte has been received; ACK has been returned
0x58	R   Data byte has been received; NOT ACK has been returned
*/
#define TWAR	((u8*)0xBA)	// TWA6 : TWA5 : TWA4 : TWA3 : TWA2 : TWA1 : TWA0 : TWGCE (page 244)
#define TWDR	((u8*)0xBB)	// 2-wire Serial Interface Data Register (page 243)
#define TWCR	((u8*)0xBC)	// TWINT : TWEA : TWSTA : TWSTO : TWWC : TWEN : x : TWIE (page 241)
#define						   TWINT													BIT(7) // Interrupt Flag (1 when done, clear on write 1)
#define								   TWEA												BIT(6) // Enable Acknowledge Bit (1 - generate ACK on data receive)
#define										  TWSTA										BIT(5) // START Condition Bit (non-auto cleared)
#define												  TWSTO								BIT(4) // STOP Condition Bit (auto cleared)
#define														  TWWC						BIT(3) // Write Collision Flag (1 when TWDR write w/TWIN=0, cleared on TWDR write w/TWIN=1)
#define																 TWEN				BIT(2) // TWI Enable Bit
#define																			TWIE	BIT(0) // TWI Interrupt Enable (1 - IRQ enabled)
#define TWAMR	((u8*)0xBD)	// TWAM6 : TWAM5 : TWAM4 : TWAM3 : TWAM2 : TWAM1 : TWAM0 : x (page 244)



#define UCSR0A	((u8*)0xC0)	// RXC0 : TXC0 : UDRE0 : FE0 : DOR0 : UPE0 : U2X0 : MPCM0 (page 195)
#define UCSR0B	((u8*)0xC1)	// RXCIE0 : TXCIE0 : UDRIE0 : RXEN0 : TXEN0 : UCSZ02 : RXB80 : TXB80 (page 196)
#define						   RXCIE0																BIT(7)
#define						            TXCIE0														BIT(6)
#define						                     UDRIE0												BIT(5)
#define						                              RXEN0										BIT(4)
#define						                                      TXEN0								BIT(3)

#define UCSR0C	((u8*)0xC2)	// UMSEL01 : UMSEL00 : UPM01 : UPM00 : USBS0 : UCSZ01/UDORD0 : UCSZ00/UCPHA0 : UCPOL0 (page 197/212)
#define UBRR0L	((u8*)0xC4)	// USART Baud Rate Register Low (page 199)
#define UBRR0H	((u8*)0xC5)	// USART Baud Rate Register High (page 199)
#define UDR0	((u8*)0xC6)	// USART I/O Data Register (page 195)


// Ports

#define GPIO_B 0
#define GPIO_C 1
#define GPIO_D 2
#define ADCPORT 3

#define IDR 0
#define DDR 1
#define ODR 2

// Port bit operations

#define _CHN(_port,_pin) (((_port)<<3)|(_pin))
#define _PORT(_chn) ((_chn)>>3)
#define _PIN(_chn) ((_chn)&0x07)
#define _MASK(_chn) (BIT(_PIN(_chn)))

#define SETBIT(nVar,nBit) ( (nVar) |=  (nBit) )
#define RESBIT(nVar,nBit) ( (nVar) &= ~(nBit) )

#define PORT_BY_NUM(nPort,nReg) (*(PRTB+(3*nPort)+(nReg)))

#define SET(nReg,nPort,nPinMask) SETBIT(PORT_BY_NUM(nPort,nReg), nPinMask)
#define RES(nReg,nPort,nPinMask) RESBIT(PORT_BY_NUM(nPort,nReg), nPinMask) // (*(PRTB+(3*nPort)+(nReg)))&=~(nPinMask)
#define GET(nReg,nPort)          (PORT_BY_NUM(nPort,nReg))

#define SETCHN(nReg,nChn)	  SET(nReg,(_PORT(nChn)),(_MASK(nChn)))
#define RESCHN(nReg,nChn)	  RES(nReg,(_PORT(nChn)),(_MASK(nChn)))
#define GETCHN(nReg,nChn)	( GET(nReg,(_PORT(nChn)))) & (_MASK(nChn) )

#define STRONG_0(bChn) RESCHN(ODR,bChn)
#define STRONG_1(bChn) SETCHN(ODR,bChn)
#define IN_MODE(bChn) RESCHN(DDR,bChn)
#define OUT_MODE(bChn) SETCHN(DDR,bChn)
#define PULLUP_ON(bChn) SETCHN(ODR,bChn)
#define PULLUP_OFF(bChn) RESCHN(ODR,bChn)
#define SOFT_STRONG_1(bChn) do { PULLUP_ON(bChn) ; OUT_MODE(bChn); } while(0)
#define SOFT_STRONG_0(bChn) do { PULLUP_OFF(bChn); OUT_MODE(bChn); } while(0)
#define WEAK_1(bChn) IN_MODE(bChn)
#define WEAK_0(bChn) OUT_MODE(bChn)
#define WEAK_OUT_MODE(bChn) do { IN_MODE(bChn); PULLUP_OFF(bChn); }	while(0) // set port as input without pullup (external 3V pull-up needed)

//#define EN_IRQ(bChan) { *PCICR |= 1<<_PORT(bChan); *(PCMSK0+_PORT(bChan)) |= _MASK(bChan); }


// #define QUE_PUT(bData,baBuf,bBufSize,bGetPointer,bCounter) { u8 bP; while ( bCounter >= bBufSize ); if ( (bP=bGetPointer+bCounter)>=bBufSize ) bP-=bBufSize; baBuf[bP]=bData; ++bCounter; }

#endif