#define __OWN__LCD1602__
#include "__HW.h"
#undef __OWN__LCD1602__

#define LCD_DUMMYREFRESHDELAY 250			// x ms

u8f fbaSpecialSympols[]={
	0x04, 0x0E, 0x1F, 0x00, 0x00, 0x00, 0x00, 0x00, // 0 - UP
	0x00, 0x00, 0x00, 0x00, 0x1F, 0x0E, 0x04, 0x00, // 1 - DOWN
	0x04, 0x0E, 0x1F, 0x00, 0x1F, 0x0E, 0x04, 0x00, // 2 - UP-DOWN
	0x00, 0x04, 0x06, 0x07, 0x06, 0x04, 0x00, 0x00, // 3 - RIGHT
	0x00, 0x04, 0x0C, 0x1C, 0x0C, 0x04, 0x00, 0x00	// 4 - LEFT
};


// small numbers (3x5) vertically oriented
u8f fbaSmallDigits[]={
	0x1F, 0x11, 0x1F,	// 0
	0x00, 0x1F, 0x00,	// 1
	0x1D, 0x15, 0x17,	// 2
	0x1F, 0x15, 0x11,	// 3
	0x1F, 0x04, 0x1C,	// 4
	0x17, 0x15, 0x1D,	// 5
	0x17, 0x15, 0x1F,	// 6
	0x18, 0x17, 0x10,	// 7
	0x1F, 0x15, 0x1F,	// 8
	0x1F, 0x15, 0x1D,	// 9
	0x1F, 0x14, 0x1F,	// A
	0x0A, 0x15, 0x1F,	// B
	0x11, 0x11, 0x1F,	// C
	0x0E, 0x11, 0x1F,	// D
	0x11, 0x15, 0x1F, // E
	0x10, 0x14, 0x1F	// F
};



u8 ba_LCD_O_Buf[LCD_BUFSIZE];		// 32 bytes
u8 ba_LCD_I_Buf[LCD_BUFSIZE];		// 32 bytes
u8 ba_CharGenBuf[8];							// chargen size = 8

u8 b_CopyPos;
u8 b_PtrPos;
u8 b_StoreCommand;
u8 b_CharGenBufSize;
u32	dw_NextCheckTime;


void LCD_4bit(u8 bNibble)
{
	SETCHN(ODR,LCD_EN );		// E=1
	PORT_BY_NUM(LCD_4BITPORT,ODR)=( PORT_BY_NUM(LCD_4BITPORT,ODR) & (~LCD_4BITMASK) ) | ( bNibble & LCD_4BITMASK ) ;
	NOP(); NOP();
	RESCHN(ODR,LCD_EN );		// E=0
}

void LCD_Command(u8 bCommand)
{
	do { *TIMSK0=OCIE0A; NOP(); *TIMSK0 =0; } while ( b_StoreCommand || b_CharGenBufSize );
	b_StoreCommand=bCommand;
	*TCCR0B = TIM0_STOP;
	*TCNT0=0;
	*OCR0A= 200-1;;
	*TCCR0B = TIM0_2M;
	*TIMSK0 =OCIE0A;
}

u8 LCD_Char(u8 bData)
{
	if ( bData ) 	{
		if ( gl_b_LCD_X >=LCD_COLUMNS ) { gl_b_LCD_X=0; gl_b_LCD_Y++; }
		if ( gl_b_LCD_Y >=LCD_ROWS    )   gl_b_LCD_Y=0;
		LCD_BUFPUT(bData);
	}
	return bData;
}

void LCD_CG(u8 bChr)
{
	LCD_Command(LCD_SETCGRAM_AC|(bChr<<3));
}

void LCD_CGF(u8 bChr, u8 bCG)
{
	u8 i=8;
	bCG<<=3; do { ba_CharGenBuf[--i]=fbaSpecialSympols[bCG++]; } while ( i ) ; LCD_CG(bChr);
}

void LCD_CGx8(u8 bChr, u8 bData)
{
	u8 bNum, i=8;
	bNum=3*LONIBBLE(bData); do { ba_CharGenBuf[--i]=fbaSmallDigits[bNum++]; } while ( i>5 ); ba_CharGenBuf[--i]=0;
	bNum=3*HINIBBLE(bData); do { ba_CharGenBuf[--i]=fbaSmallDigits[bNum++]; } while ( i ); ba_CharGenBuf[i]=0;
	LCD_CG(bChr);
}

void LCD_CGx16(u8 bChr, u16 wData)
{
	LCD_CGx8(bChr, LOBYTE(wData));
	LCD_CGx8(bChr+1, HIBYTE(wData));
}

void LCD_CGx32(u8 bChr, u32 dwData)
{
	LCD_CGx16(bChr, LOWORD(dwData));
	LCD_CGx16(bChr+2, HIWORD(dwData));
}


void LCD_Clear(void)
{
	gl_b_LCD_X=LCD_BUFSIZE; do { ba_LCD_I_Buf[--gl_b_LCD_X]=' '; } while (gl_b_LCD_X);
	gl_b_LCD_Y=0;
}

void LCD_ClearToEOL(void)
{
	u8 bTmp=gl_b_LCD_X; while ( gl_b_LCD_X<LCD_COLUMNS ) LCD_BUFPUT(' '); gl_b_LCD_X=bTmp;
}

#pragma vector=IRQ_TIM0CA
__interrupt void fIRQ_TIM0CA(void)
{
	u8 bSym, bTmp;

	*TCCR0B = TIM0_STOP;
	*TCNT0 = 0;

	if ( b_CharGenBufSize ) { bSym=ba_CharGenBuf[--b_CharGenBufSize]; SETCHN(ODR,LCD_RS); goto jSendData; }

	if ( bSym=b_StoreCommand ) {
		b_StoreCommand=0;
jSendCommand:;
		if      ( bSym & LCD_SETDDRAM_AC )  { b_PtrPos=bSym; }
		else if ( bSym & LCD_SETCGRAM_AC ) 	{ b_PtrPos=bSym; b_CharGenBufSize=8; }
		RESCHN(ODR,LCD_RS);
		goto jSetAutoIRQ;
	}

 	if ( gl_dwSysTime >= dw_NextCheckTime ) {
jNextChar:;
		if ( b_CopyPos < LCD_BUFSIZE ) {
			bSym=ba_LCD_I_Buf[b_CopyPos];
			if ( ba_LCD_O_Buf[b_CopyPos]==bSym ) { ++b_CopyPos; goto jNextChar; }
			bTmp=LCD_SETDDRAM_AC | (b_CopyPos&0x0F) | ((b_CopyPos&0x10)<<2) ;
			if ( b_PtrPos!= bTmp ) { bSym=bTmp; goto jSendCommand; }
			ba_LCD_O_Buf[b_CopyPos++]=bSym;
jSendData:;
			++b_PtrPos;
			SETCHN(ODR,LCD_RS);
jSetAutoIRQ:;
			LCD_4bit(bSym); LCD_4bit(bSym<<4);
			*OCR0A =  200-1; *TCCR0B = TIM0_2M;			// 100us
			goto jIRQRet;
		}
		b_CopyPos=0;
		dw_NextCheckTime=gl_dwSysTime+100L;
	}
	*OCR0A =  156-1; *TCCR0B = TIM0_15625;			// 15.625/1ms
jIRQRet:;
}

void LCD_init(void)
{
	gl_psPrintChar=&LCD_Char;
	gl_b_LCD_X=LCD_BUFSIZE;	do { ba_LCD_O_Buf[--gl_b_LCD_X]=0; } while (gl_b_LCD_X);
	LCD_Clear(); gl_b_LCD_X=(LCD_COLUMNS-16)/2; _s_sF("= StarScan 0.1 =");


	*TCCR0B = TIM0_STOP;		// stop LCD timer
	*TCCR0A = TIM0_A_CTC_OCR0A;	// mode "Clear Timer on Compare"
	*TIMSK0 = 0;

	SETCHN(DDR,LCD_EN );	// 	- LCD EN : out
	SETCHN(DDR,LCD_RS);		//  - LCD RS : out
	SET(DDR,LCD_4BITPORT,LCD_4BITMASK);	// LCD_4BITPORT : out
	SETCHN(ODR,LCD_EN );	//	- LCD EN = 1 : hold
	RESCHN(ODR,LCD_RS );	// 	- LCD RS = 0 : command

	while ( gl_dwSysTime<20L );	// wait 20 mS after powerUp
	LCD_4bit(0x30); delay_100us(45);
	LCD_4bit(0x30); delay_100us(45);
	LCD_4bit(0x30); delay_100us(2);
	LCD_4bit(0x20); delay_100us(2);
	b_StoreCommand=0;
	b_CharGenBufSize=0;
	b_CopyPos=LCD_BUFSIZE;
	dw_NextCheckTime=0L;
	LCD_Command(0x28);
	LCD_Command(0x0C);
	LCD_Command(0x06);
	delay_ms(100);
	BUZZER(200);
	delay_ms(1000);
	LCD_Clear();
	BUZZER(100);
}
